/*
* Copyright (C) 2005, Pasi P. Liimatainen
* Distributable under GNU General Public License (v2)
* See terms of license at http://www.gnu.org
*/

function GetTagsRequester(aParent) {
  this.parent = aParent;
  // Executed when the URL has been got.
  this.asyncGetURLResponse = function(resp) {
    if (resp != null) {
      var root = resp.documentElement;
      var myTags = root.getElementsByTagName("tag");
      for (var i = 0; i < myTags.length; i++) {
        var tagName = myTags[i].getAttribute("tag");
        var tagCount = myTags[i].getAttribute("count");
        this.parent.tags[tagName] = parseInt(tagCount);
        this.parent.tagNames.push(tagName);
      }
    }
    this.callback(this.callbackParam);
  };
  // Process error.
  this.asyncGetURLError = function(error) {
    return;  //TODO Add error message.
  };
}

function Tags(aDeliciousService) {
  this.tags = {};
  this.tagNames = [];
  this.deliciousService = aDeliciousService;

  this.getTagsRequester = new GetTagsRequester(this);
}

Tags.prototype = {
  preloadTags: function(aCallback, aParam, aUsername, aPassword) {
    this.getTagsRequester.callback = aCallback;
    this.getTagsRequester.callbackParam = aParam;
    if (this.tagNames.length == 0) {
      this.deliciousService.getTagsForUser(this.getTagsRequester, aUsername, aPassword);
    } else {
      if (aCallback != null) {
        aCallback(aParam);
      }
    }
  },

  getTags: function() {
    if (this.tagNames.length == 0) {
      const xmlDOM = this.deliciousService.getTags();
      var root = xmlDOM.documentElement;
      var myTags = root.getElementsByTagName("tag");
      for (var i = 0; i < myTags.length; i++) {
        var tagName = myTags[i].getAttribute("tag");
        var tagCount = myTags[i].getAttribute("count");
        this.tags[tagName] = parseInt(tagCount);
        this.tagNames.push(tagName);
      }
    }
    return this.tags;
  },
  getTagNames: function() {
    if (this.tagNames.length == 0) {
      this.getTags();
    }
    return this.tagNames;
  },
  getTagNamesSorted: function() {
    if (this.tagNames.length == 0) {
      this.getTags();
    }
    var lower = [];
    var map = {};
    for (t in this.tags) {
      map[t.toLowerCase()] = t;
      lower.push(t.toLowerCase());
    }
    lower = lower.sort();
    var sorted = [];
    for (t in lower) {
      sorted.push(map[lower[t]]);
    }
    return sorted;
  },
  addTags: function(tags) {
    var usersTags = this.getTags();
    for (t in tags) {
      if (usersTags[tags[t]] == null) {
        usersTags[tags[t]] = 1;
        this.tagNames.push(tags[t]);
      } else {
        usersTags[tags[t]] += 1;
      }
    }
  },
  asXml: function(aSorted) {
    if (this.tagNames.length == 0) {
      this.getTags();
    }
    var tagXML = '<tags>';
    var tags = this.tags;
    var tagNames = this.tagNames;
    var lower = [];
    var map = {};
    for (t in tags) {
      map[t.toLowerCase()] = t;
      lower.push(t.toLowerCase());
    }
    tagNames = aSorted ? lower.sort() : lower;
    for (t in tagNames) {
      tagXML += '<tag count="' + tags[map[tagNames[t]]] + '" tag="'+map[tagNames[t]]+'"/>';
    }
    tagXML += '</tags>';
    return tagXML;
  },
  reload: function(aCallback, aUsername, aPassword) {
    this.tags = {};
    this.tagNames = [];
    this.preloadTags(aCallback, 0, aUsername, aPassword);
  }
}
